<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="xml" omit-xml-declaration="no"/>
	
	<xsl:variable name="viewer">/tools/explore/</xsl:variable>
	<xsl:variable name="imgbase">/nkse/style/img/</xsl:variable>
	
	 <xsl:include href="res:/nkse/style/xsl/format_resolution.xsl"/>	
	
	<xsl:strip-space elements="n"/>
	
<xsl:template match="/">
	<xsl:call-template name="exception"/>
</xsl:template>
	
<xsl:template name="exception">
	<xsl:param name="omit-head" select="'true'"></xsl:param>
	
	<div id="exception-wrapper">
		
		<script language="JavaScript">
		<xsl:comment>
		function showException()
		{	$('#exception').toggle();
		}
		function showJava(id)
		{	$(id).toggle();
		}
		function jumpToLowestFrame()
		{	$('#exception').show();
			id=$('.exception-block').last().attr("id");
			document.location="#"+id;
		}
		function explore(hash)
		{	hostname=window.location.hostname;
			url="http://"+hostname+":1060/tools/explore/"+hash;
			document.location=url;
		}
		
		</xsl:comment>
		</script>
		<link type="text/css" href="/tools/ae/pub/explore.css" rel="stylesheet"/>
		<style type="text/css">
			#exception-wrapper { padding: 10px; }
			#exception-wrapper a:link { text-decoration: underline; }
			#exception { line-height: normal; margin-top: 1em;  }
			#exception td { padding: 0; }
			#exception a:link { text-decoration: none; }
			.headline-id { font-size: x-large; color: #CC0000; font-weight: bold }
			.headline-message { }
			
			.request { text-align: center;}
			.endpoint { background: #F5F5F5; border: 1px solid #CCCCCC; border-radius: 4px 4px 0 0; padding: 4px; }
			.endpoint .title { text-align: center; }
			.exception-block { background: #F2DEDE; color: #B94A48; border: 1px solid #EBCCD1; border-radius: 0 0 4px 4px; padding: 4px; }
			.exception-id { font-weight: bold; }
			.java { display:none; padding: 0.5em; background: white; font-family: monospace; font-size: 90%; color: black; border-radius: 4px; border: 1px solid #EBCCD1;}
			.resolution-block { background: #E0C0F0; color: #402080; border: 1px solid #B0A0E0; border-radius: 4px; padding: 4px; }
			.resolution-detail { padding: 0.5em; background: white; color: black; border-radius: 4px; border: 1px solid #B0A0E0;}
		</style>
	
		<xsl:choose>
			<xsl:when test="$omit-head='true'">
				<xsl:call-template name="main"/>
			</xsl:when>
			<xsl:otherwise>
				<table border="0" cellspacing="0" cellpadding="2">
					<tr>
						<td valign="top"><img src="{$imgbase}exception.png"/></td>
						<td>
							<xsl:variable name="deepest" select="(descendant-or-self::ex[id])[last()]"/>
							<xsl:variable name="id" select="$deepest/id"/>
							<xsl:variable name="message" select="$deepest/message"/>
							<p>
								<xsl:choose>
									<xsl:when test="$id='Request Resolution Failure' and (descendant-or-self::verb)[last()]='SOURCE'">
										<div class="headline-id"><xsl:value-of select="$id"/></div>
										<div style="font-weight: bold;">
											<xsl:call-template name="wrapped">
												<xsl:with-param name="text" select="(descendant-or-self::requestDecoded)[last()]"/>
											</xsl:call-template>
										</div>
									</xsl:when>
									<xsl:when test="$id='com.ten60.netkernel.util.NetKernelException'">
										<div class="headline-id"><xsl:value-of select="$message"/></div>
									</xsl:when>
									<xsl:otherwise>
										<div class="headline-id"><xsl:value-of select="$id"/></div>
										<xsl:call-template name="wrapped">
											<xsl:with-param name="text" select="$message"/>
										</xsl:call-template>
									</xsl:otherwise>
								</xsl:choose>
							</p>
							<xsl:call-template name="main"/>
						</td>
					</tr>
				</table>
			</xsl:otherwise>
		</xsl:choose>
	</div>
	</xsl:template>	
	
	<xsl:template name="main">
		<p>
			An unhandled exception occurred whilst processing the request.
		</p>
		
		<div>
			<button class="btn btn-primary" onclick="javascript:jumpToLowestFrame()">View Request Trace</button>
			<xsl:text> </xsl:text>
			<button class="btn btn-default" onclick="location.reload(true)"><i class="icon-refresh"><xsl:text> </xsl:text></i> Retry Request</button>
		</div>
		
		<div id="exception" style="display: none;">
			<table>
				<xsl:apply-templates select="ex/ex"/>
			</table>
		</div>
	</xsl:template>
	
	<xsl:template match="ex">
		<tr><td>
		<xsl:choose>
		
			<xsl:when test="id='RequestFrameException' or id='RequestFrameError'">
				<div class="request">
					<div><img src="{$imgbase}request-arrow-tail.png"/></div>
					<div class="request">
					<xsl:choose>
						<xsl:when test="requestDecoded and verb='SOURCE'">
							<div style="margin-bottom: 0.25em;"><xsl:value-of select="verb"/></div>
							<div style="font-weight: bold; margin-bottom: 0.25em;">
								<xsl:choose>
									<xsl:when test="substring(requestDecoded, 1, 6) = 'active'">
										<xsl:call-template name="split">
											<xsl:with-param name="text"><xsl:value-of select="requestDecoded"/></xsl:with-param>
											<xsl:with-param name="first"><true/></xsl:with-param>
										</xsl:call-template>
									</xsl:when>
									<xsl:otherwise>
										<xsl:call-template name="wrapped">
											<xsl:with-param name="text"><xsl:value-of select="requestDecoded"/></xsl:with-param>
										</xsl:call-template>
									</xsl:otherwise>
								</xsl:choose>
							</div>
						</xsl:when>
						<xsl:otherwise>
						<div style="font-weight: bold; margin-bottom: 0.25em;">
							<xsl:call-template name="wrapped">
								<xsl:with-param name="text"><xsl:value-of select="request"/></xsl:with-param>
							</xsl:call-template>
						</div>
						</xsl:otherwise>
					</xsl:choose>
					</div>
					<xsl:if test="asClass">
						<a href="{$viewer}{asClass}">link</a>
					</xsl:if>
					<div><img src="{$imgbase}request-arrow-head.png"/></div>
				</div>	
			</xsl:when>
			
			<xsl:when test="id='SubrequestException' or id='SubrequestError'">
				<div class="endpoint">
					<div class="title">
						<xsl:choose>
							<xsl:when test="spaceHash">
							    <a href="javascript:explore('{spaceHash}')">
									<span class="ui-icon-ae ui-icon-ae-space"> </span>
									<xsl:value-of select="space"/>
								</a>
							</xsl:when>
							<xsl:otherwise>
								<span class="ui-icon-ae ui-icon-ae-space"> </span>
								<xsl:value-of select="space"/>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:text> / </xsl:text>
					    <xsl:choose>
							<xsl:when test="endpointHash">
								<a href="javascript:explore('{endpointHash}')" title="Display in Space Explorer">
									<span class="ui-icon-ae ui-icon-ae-physical"> </span>
									<xsl:value-of select="endpoint"/>
								</a>
							</xsl:when>
							<xsl:otherwise>
								<span class="ui-icon-ae ui-icon-ae-physical"> </span>
								<xsl:value-of select="endpoint"/>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:text> / </xsl:text>
						<xsl:choose>
							<xsl:when test="logicalEndpointHash">
								<a href="javascript:explore('{logicalEndpointHash}')">
									<span class="ui-icon-ae ui-icon-ae-ep"> </span>
									<xsl:value-of select="logicalEndpoint"/>
								</a>
							</xsl:when>
							<xsl:otherwise>
								<span class="ui-icon-ae ui-icon-ae-ep"> </span>
								<xsl:value-of select="logicalEndpoint"/>
							</xsl:otherwise>
						</xsl:choose>
					</div>
					<xsl:apply-templates select="ex"/>
				</div>				
			</xsl:when>
			
			<xsl:when test="id!='org.ten60.netkernel.layer0.nkf.NKFException' and not(trace)">
				<xsl:variable name="id" select="generate-id()"/>
				<div class="exception-block" id="{$id}">

					<xsl:if test="stack" >
						<div style="float: right;">
							<a class="btn btn-xs btn-primary" href="javascript:showJava('#java{$id}');">View Java stacktrace</a>
						</div>
					</xsl:if>
					
					<div class="exception-id"><xsl:value-of select="id"/></div>
					
					<xsl:if test="string-length(message)">
						<div><xsl:value-of select="message"/></div>
					</xsl:if>
					<xsl:if test="location" >
						<div><xsl:value-of select="location"/></div>
					</xsl:if>
					
					<xsl:if test="stack" >
						<div class="java" id="java{$id}">
							<xsl:apply-templates select="stack/level"/>
						</div>
					</xsl:if>
				</div>
				
			</xsl:when>
			
			<xsl:when test="id!='org.ten60.netkernel.layer0.nkf.NKFException' and trace">
				<xsl:variable name="id" select="generate-id()"/>
				<div class="resolution-block" id="{$id}">

					<div class="exception-id"><xsl:value-of select="id"/></div>
					
					<xsl:if test="trace">
						<xsl:for-each select="trace">
							<div class="resolution-detail">
								<xsl:call-template name="resolution"/>
							</div>
						</xsl:for-each>
					</xsl:if>
				</div>
				
			</xsl:when>
			
		</xsl:choose>
		</td></tr>		
	</xsl:template>	
		 
		 
	<xsl:template match="level">
		<div class="error-javastack-level"><xsl:value-of select="."/></div>
	</xsl:template>

	<xsl:template match="s">
		<div><xsl:value-of select="."/></div>
	</xsl:template>
	
	<xsl:template name="wrapped">
		<xsl:param name="text"/>
		<xsl:choose>
			<xsl:when test="string-length($text)>60">
				<xsl:value-of select="substring($text,1,60)"/><wbr/><xsl:call-template name="wrapped">
					<xsl:with-param name="text" select="substring($text,60+1)"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$text"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="split">
		<xsl:param name="text"/>
		<xsl:param name="first"/>
		<xsl:if test="string-length($text)">
			<xsl:if test="not($first)">
				<br />+
			</xsl:if>
			<xsl:value-of select="substring-before(concat($text,'+'),'+')"/>
			<xsl:call-template name="split">
				<xsl:with-param name="text" select="substring-after($text, '+')"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
		

</xsl:stylesheet>
