package org.netkernel.cp;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.w3c.dom.Document;

public class PanelAccessor extends StandardAccessorImpl
{
	public PanelAccessor()
	{	declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		ControlPanelConfig config = aContext.source("active:controlPanelConfig",ControlPanelConfig.class);
		String panelId=aContext.getThisRequest().getArgumentValue("id");
		IHDSNode panels=config.getPanels();
		IHDSNode panel=config.getPanel(panelId);
		if (panel==null)
		{	throw aContext.createFormattedException("EX_UNKNOWN_PANEL", "MSG_UNKNOWN_PANEL", null, null, panelId);
		}
		
		//style tabs
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator","res:/nkse/style/xsl/styleCPTabs.xsl");
		req.addArgumentByValue("operand",panels);
		req.addArgumentByValue("wide",false);
		req.addArgumentByValue("panelId",panelId);
		req.setRepresentationClass(Document.class);
		req.setRepresentationClass(Document.class);
		INKFResponseReadOnly tabs=aContext.issueRequestForResponse(req);
		
		//style panel
		req=aContext.createRequest("active:xslt");
		req.addArgument("operator","res:/nkse/style/xsl/styleCPPanel.xsl");
		req.addArgumentByValue("panelId",panelId);
		req.addArgumentByValue("operand", panel);
		req.setRepresentationClass(Document.class);
		INKFResponseReadOnly content=aContext.issueRequestForResponse(req);
		
		//wrap with standard template
		String name=panel.getFirstValue("/panel/title").toString();
		String subtitle=panel.getFirstValue("/panel/subtitle").toString();
		String icon=(String)panel.getFirstValue("/panel/icon");
		if (icon==null)
		{	icon="/nkse/style/img/noicon.png";
		}
		req=aContext.createRequest("active:xrl2");
		req.addArgument("tolerant", "true");
		req.addArgument("template", "res:/nkse/style/template/cp-template.xml");
		req.addArgumentByValue("pagetitle","NetKernel - "+name);
		req.addArgumentByValue("tooltitle",name);
		req.addArgumentByValue("subtitle",subtitle);
		req.addArgumentByValue("icon",icon);
		req.addArgumentFromResponse("tabs", tabs);
		req.addArgumentFromResponse("content", content);
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		
		INKFResponse respOut=aContext.createResponseFrom(resp);
		respOut.setMimeType("text/html");
	}
}
